@@ -93,11 +93,22 @@ start_format (void)
 static void
 init_commands (void)
 {
+	const char *path;
+	char *newpath;
+
 	floppy.mke2fs_cmd = NULL;
 	floppy.mformat_cmd = NULL;
 	floppy.badblocks_cmd = NULL;
 
-	putenv ("PATH=$PATH:/sbin:/usr/sbin:/usr:/usr/bin");
+	path = g_getenv ("PATH");
+
+	if (path)
+		newpath = g_strconcat ("PATH=", path, ":/sbin:/usr/sbin:/usr:/usr/bin", NULL);
+	else
+		newpath = g_strdup ("PATH=/sbin:/usr/sbin:/usr:/usr/bin");
+
+	putenv (newpath); /* Sigh, we have to leak it */
+
 	floppy.mke2fs_cmd    = g_find_program_in_path ("mke2fs");
 	floppy.mformat_cmd   = g_find_program_in_path ("mformat");
 	floppy.badblocks_cmd = g_find_program_in_path ("mbadblocks");
@@ -335,7 +346,30 @@ main (int argc, char *argv[])
 			break;
 
 		if (button == GFLOPPY_HELP) {
-			gnome_help_display ("gfloppy", NULL, NULL);
+			GError *error;
+
+			error = NULL;
+			gnome_help_display ("gfloppy", NULL, &error);
+			if (error) {
+				GtkWidget *dialog;
+
+				dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+								 0,
+								 GTK_MESSAGE_ERROR,
+								 GTK_BUTTONS_CLOSE,
+								 _("Could not display help for the "
+								   "floppy formatter.\n"
+								   "%s"),
+								 error->message);
+
+				g_signal_connect_swapped (dialog, "response",
+							  G_CALLBACK (gtk_widget_destroy),
+							  dialog);
+				gtk_widget_show (dialog);
+
+				g_error_free (error);
+			}
+
 			continue;
 		}
 		density_option = glade_xml_get_widget (xml, "density_option");
