@@ -14,6 +14,8 @@ static GtkWidget *app = NULL;
 static GtkWidget *icon_list = NULL;
 static GtkWidget *status_label = NULL;
 static GtkWidget *compress_button = NULL;
+static GtkWidget *app_bar = NULL;
+static GtkWidget *progress_bar = NULL;
 static GHashTable *file_ht = NULL;
 static gint number_of_files = 0;
 static gint number_of_dirs = 0;
@@ -30,10 +32,12 @@ static gchar *filename = NULL;
 static pid_t temporary_pid = 0;
 static gchar *temporary_file = NULL;
 struct poptOption options;
+struct argv_adder argv_adder;
 
 static GtkWidget *create_archive_widget=NULL;
 static GtkWidget *remove_widget=NULL;
 static GtkWidget *clear_widget=NULL;
+static GtkWidget *selectall_widget=NULL;
 
 #define ERRDLG(error) gtk_message_dialog_new (GTK_WINDOW(app), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error);
 #define ERRDLGP(error,parent) gtk_message_dialog_new (GTK_WINDOW(parent), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error);
@@ -145,6 +149,7 @@ static void
 remove_pos (gint pos)
 {
 	File *f;
+	gchar *status_msg;
 
 	f = gnome_icon_list_get_icon_data (GNOME_ICON_LIST (icon_list), pos);
 	gnome_icon_list_remove (GNOME_ICON_LIST (icon_list), pos);
@@ -156,12 +161,18 @@ remove_pos (gint pos)
 	else if (f->type == TYPE_FOLDER)
 		number_of_dirs --;
 	g_hash_table_remove (file_ht, f->base_name);
+        
+ 	status_msg = g_strdup_printf(_("%s removed"), f->base_name);
+        gnome_appbar_push (GNOME_APPBAR(app_bar), 
+                           status_msg);
+        g_free (status_msg);
 	free_file (f);
 
 	if (number_of_files + number_of_dirs <= 0) {
 		gtk_widget_set_sensitive (compress_button, FALSE);
 		gtk_widget_set_sensitive (create_archive_widget, FALSE);
 		gtk_widget_set_sensitive (clear_widget, FALSE);
+		gtk_widget_set_sensitive (selectall_widget, FALSE);
 	}
 
 	update_status ();
@@ -173,15 +184,15 @@ struct argv_adder {
 	gint pos;
 };
 
+
 static void
 make_argv_fe (gpointer key, gpointer value, gpointer user_data)
 {
 	File *f = value;
-	struct argv_adder *argv_adder = user_data;
-	gchar *file = g_build_filename ((argv_adder->link_dir), (f->base_name), NULL);
-
-	argv_adder->argv[argv_adder->pos ++] = g_strconcat ("\"", f->base_name, "\"", NULL);
-
+	gchar *file;
+	struct argv_adder *argv_adding = user_data;
+	file = g_build_filename ((argv_adding->link_dir), (f->base_name), NULL);
+	argv_adding->argv[argv_adding->pos ++] = f->base_name;
 	/* FIXME: catch errors */
 	symlink (f->name, file);
 }
@@ -197,10 +208,32 @@ whack_links_fe (gpointer key, gpointer value, gpointer user_data)
 	g_free (file);
 }
 
+static gint
+update_progress_bar (gpointer p)
+{
+        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (progress_bar));
+
+        return TRUE;
+}
+
+static void
+start_progress_bar (gboolean flag)
+{
+	static gint timeout;
+
+	if (flag)
+		timeout = gtk_timeout_add (100, update_progress_bar, NULL);
+	else {
+		gtk_timeout_remove (timeout);
+        	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 0.0);
+	}
+}
+
 static void
 setup_busy (GtkWidget *w, gboolean busy)
 {
 	GdkCursor *cursor;
+	static gint timeout;
 
 	if (busy) {
 		/* Change cursor to busy */
@@ -215,14 +248,37 @@ setup_busy (GtkWidget *w, gboolean busy)
 }
 
 static gboolean
-create_archive (const gchar *fname,
+handle_io (GIOChannel *ioc, GIOCondition condition, gpointer data) 
+{
+	start_progress_bar (FALSE);
+
+	g_hash_table_foreach (file_ht, whack_links_fe, argv_adder.link_dir);
+
+	/* FIXME: handle errors */
+	rmdir (argv_adder.link_dir);
+	g_free (argv_adder.link_dir);
+	g_free (argv_adder.argv);
+	cleanup_tmpstr ();
+}
+
+
+static gboolean
+create_archive (GtkWidget *fsel,
+		 const gchar *fname,
 		const gchar *dir,
 		gboolean gui_errors)
 {
+	GIOChannel *ioc;
 	gboolean status = TRUE;
-	pid_t pid;
-	struct argv_adder argv_adder;
 	gchar *tmpfname = NULL;
+	gint child_pid;
+	gint child_fd;
+
+	
+	if (fsel) {
+		setup_busy (fsel, FALSE);
+		gtk_widget_destroy (fsel);
+	} 
 
 	if (dir == NULL)
 		argv_adder.link_dir = make_temp_dir ();
@@ -251,6 +307,8 @@ create_archive (const gchar *fname,
 	filestr = g_strjoinv(" ", argv_adder.argv);
 	gzipstr = g_strconcat(" | ", gzip_prog, " - > ", (gchar *)fname, NULL);
 
+	filestr = g_strescape (filestr, NULL);
+
 	argv_adder.argv[0] = sh_prog;
 	argv_adder.argv[1] = "-c";
 	argv_adder.argv[2] = g_strconcat(tarstr, filestr, gzipstr, NULL);	
@@ -258,40 +316,27 @@ create_archive (const gchar *fname,
 
 	/* FIXME: get error output, check for errors, etc... */
 
-	pid = fork ();
-	if (pid < 0) {
-		if (gui_errors) {
-			ERRDLG (_("Cannot start the archive (tar) program"));
-		} else {
-			fprintf (stderr,
-				 _("Cannot start the archive (tar) program"));
-		}
+	/* setup_busy (app, TRUE); */
+	start_progress_bar (TRUE);
+	if (!g_spawn_async_with_pipes (argv_adder.link_dir, argv_adder.argv, 
+				       NULL, (G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD),
+		                       NULL, NULL, &child_pid, NULL, &child_fd, NULL, NULL)) {
 		status = FALSE;
-	} else if (pid == 0) {
-		umask (022);
-		/* FIXME: handler errors */
-		chdir (argv_adder.link_dir);
-		execv (sh_prog, argv_adder.argv);
-		fprintf (stderr, _("Cannot run tar/gtar"));
-		cleanup_tmpstr ();
-		_exit (1);
+		if (gui_errors) {
+			gchar *tmp = NULL;
+			tmp = g_strdup_printf (_("Failed to create %s"), filestr);
+			ERRDLG (tmp);
+			g_free (tmp);
+		}
+		else
+			fprintf (stderr, _("Failed to create %s"), filestr);
 	}
+	ioc = g_io_channel_unix_new (child_fd);
+        g_io_add_watch (ioc, G_IO_HUP,
+                        handle_io, NULL);
+        g_io_channel_unref (ioc);
+/*	setup_busy (app, FALSE);  */
 
-	/* FIXME: Do some progress bar or what not rather then block */
-	if (pid > 0)
-		waitpid (pid, 0, 0);
-
-	/* FIXME: handle errors */
-	g_hash_table_foreach (file_ht, whack_links_fe, argv_adder.link_dir);
-
-	if (dir == NULL) {
-		/* FIXME: handle errors */
-		rmdir (argv_adder.link_dir);
-		g_free (argv_adder.link_dir);
-	}
-
-	g_free (argv_adder.argv);
-	cleanup_tmpstr ();
 	return status;
 }
 
@@ -329,7 +374,7 @@ start_temporary (void)
 	temporary_pid = fork ();
 
 	if (temporary_pid == 0) {
-		if ( ! create_archive (file, dir, FALSE /* gui_errors */)) {
+		if ( ! create_archive (NULL, file, dir, FALSE /* gui_errors */)) {
 			_exit (1);
 		} else {
 			_exit (0);
@@ -339,7 +384,7 @@ start_temporary (void)
 	/* can't fork? don't dispair, do synchroniously */
 	if (temporary_pid < 0) {
 		setup_busy (app, TRUE);
-		if ( ! create_archive (file, dir, TRUE /* gui_errors */)) {
+		if ( ! create_archive (NULL, file, dir, TRUE /* gui_errors */)) {
 			setup_busy (app, FALSE);
 			g_free (file);
 			g_free (dir);
@@ -477,6 +522,8 @@ select_all_cb (GtkWidget *w, gpointer data)
 	for (i = 0; i < num_icons; i++) {
 		gnome_icon_list_select_icon (GNOME_ICON_LIST (icon_list), i);
 	}
+        gnome_appbar_push (GNOME_APPBAR (app_bar), 
+                           _("All files selected"));
 }
 
 static gboolean
@@ -493,10 +540,15 @@ clear_cb (GtkWidget *w, gpointer data)
 	g_hash_table_foreach_remove (file_ht, file_remove_fe, NULL);
 	number_of_files = number_of_dirs = 0;
 	gnome_icon_list_clear (GNOME_ICON_LIST (icon_list));
+
+        gnome_appbar_push (GNOME_APPBAR (app_bar), 
+                           _("All files removed"));
+
 	gtk_widget_set_sensitive (compress_button, FALSE);
 	gtk_widget_set_sensitive (create_archive_widget, FALSE);
 	gtk_widget_set_sensitive (remove_widget, FALSE);
 	gtk_widget_set_sensitive (clear_widget, FALSE);
+	gtk_widget_set_sensitive (selectall_widget, FALSE);
 	update_status ();
 }
 
@@ -535,6 +587,7 @@ remove_cb (GtkWidget *w, gpointer data)
 		gtk_widget_set_sensitive (compress_button, FALSE);
 		gtk_widget_set_sensitive (create_archive_widget, FALSE);
 		gtk_widget_set_sensitive (clear_widget, FALSE);
+		gtk_widget_set_sensitive (selectall_widget, FALSE);
 	}
 }
 
@@ -542,6 +595,8 @@ static void
 save_ok (GtkWidget *widget, GtkFileSelection *fsel)
 {
 	gchar *fname;
+        gchar *status_msg;
+        gchar *tmpfname = NULL;
 
 	g_return_if_fail (GTK_IS_FILE_SELECTION(fsel));
 
@@ -556,19 +611,32 @@ save_ok (GtkWidget *widget, GtkFileSelection *fsel)
 		   ! query_dialog (_("File exists, overwrite?"))) {
 		setup_busy (GTK_WIDGET (fsel), FALSE);
 		return;
-	} else if ( ! create_archive (fname,
+	} else if ( ! create_archive (GTK_WIDGET (fsel),
+				      fname,
 				      NULL /*dir*/,
 				      TRUE /* gui_errors */)) {
 		/* the above should do error dialog itself */
-		setup_busy (GTK_WIDGET (fsel), FALSE);
 		return;
 	}
 
-	setup_busy (GTK_WIDGET (fsel), FALSE);
 
+	/* Added code for appending .tar.gz to archive file */
+	tmpfname = g_strstr_len ((gchar *)fname, strlen((gchar *)fname), ".tar.gz");
+	if (tmpfname == NULL)
+		tmpfname = g_strstr_len ((gchar *)fname, strlen((gchar *)fname), ".tgz");
+	if (tmpfname != NULL) {
+		if (!((g_ascii_strcasecmp(tmpfname, ".tar.gz") == 0) || (g_ascii_strcasecmp(tmpfname, ".tgz") == 0)))
+			fname = g_strconcat ((gchar *)fname, ".tar.gz", NULL);
+	} else {
+		fname = g_strconcat ((gchar *)fname, ".tar.gz", NULL);
+	}
+
+        status_msg = g_strdup_printf(_("%s created"), g_path_get_basename(fname));
 	g_free (filename);
 	filename = g_strdup (fname);
-	gtk_widget_destroy (GTK_WIDGET (fsel));
+        gnome_appbar_push (GNOME_APPBAR (app_bar), 
+                           status_msg);
+       g_free (status_msg);
 }
 
 static void
@@ -611,6 +679,7 @@ add_file (const gchar *file)
 	gint i;
 	const gchar *base_name;
 	gchar *fullname;
+        gchar *status_msg;
 
 	if (file == NULL ||
 	    file[0] == '\0')
@@ -674,6 +743,9 @@ add_file (const gchar *file)
 	gtk_widget_set_sensitive (compress_button, TRUE);
 	gtk_widget_set_sensitive (create_archive_widget, TRUE);
 	gtk_widget_set_sensitive (clear_widget, TRUE);
+	gtk_widget_set_sensitive (selectall_widget, TRUE);
+
+        filename = g_strdup_printf("%s/",g_path_get_dirname(fullname));
 
 	f = g_new0 (File, 1);
 	f->type = type;
@@ -694,9 +766,12 @@ add_file (const gchar *file)
 	gnome_icon_list_set_icon_data (GNOME_ICON_LIST (icon_list), pos, f);
 
 	g_hash_table_insert (file_ht, f->base_name, f);
-
+    
 	update_status ();
-
+        status_msg = g_strdup_printf(_("%s added"), base_name);
+        gnome_appbar_push (GNOME_APPBAR (app_bar), 
+                           status_msg);
+        g_free (status_msg);
 	g_free (fullname);
 }
 
@@ -792,33 +867,83 @@ drag_data_get (GtkWidget          *widget,
 				8, string, strlen (string)+1);
 	g_free (string);
 }
+GtkWidget*
+archive_button_new_with_image (const gchar* text, const gchar* image_path)
+{
+        GtkWidget *button;
+        GtkStockItem item;
+        GtkWidget *label;
+        GtkWidget *image;
+        GtkWidget *hbox;
+        GtkWidget *align;
+
+        button = gtk_button_new ();
+
+        if (GTK_BIN (button)->child)
+                gtk_container_remove (GTK_CONTAINER (button),
+                                      GTK_BIN (button)->child);
+
+	if (image) {
+                label = gtk_label_new_with_mnemonic (text);
+
+                gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+                image = gtk_image_new_from_file (image_path);
+                hbox = gtk_hbox_new (FALSE, 2);
+
+                align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+                gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+                gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+                gtk_container_add (GTK_CONTAINER (button), align);
+                gtk_container_add (GTK_CONTAINER (align), hbox);
+                gtk_widget_show_all (align);
+
+                return button;
+        }
+
+        label = gtk_label_new_with_mnemonic (text);
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+        gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+        gtk_widget_show (label);
+        gtk_container_add (GTK_CONTAINER (button), label);
+
+        return button;
+}
+
 
 /* Menus */
 static GnomeUIInfo file_menu[] = {
-	GNOMEUIINFO_ITEM_NONE
+        GNOMEUIINFO_ITEM_STOCK 
 		(N_("_Add file or folder..."),
-		 N_("Add a file or folder to the archive"),
-		 add_cb),
-	GNOMEUIINFO_ITEM_NONE
-		(N_("_Create archive..."),
-		 N_("Create a new archive from the items"),
-		 archive_cb),
+                 N_("Add a file or folder to archive"),
+                 add_cb, GTK_STOCK_DND_MULTIPLE),
+
+	{ GNOME_APP_UI_ITEM, N_("_Create archive..."), N_("Create a new archive from the items"), 
+	(gpointer) archive_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, "document-icons/gnome-compressed.png", 
+	0, (GdkModifierType) 0, NULL },
+
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_EXIT_ITEM (quit_cb, NULL),
+	GNOMEUIINFO_MENU_QUIT_ITEM (quit_cb, NULL),
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo edit_menu[] = {
 	GNOMEUIINFO_ITEM_NONE (N_("_Remove"),
-			       N_("Remove the selected item(s)"),
+			       N_("Remove the selected item"),
 			       remove_cb),
-	GNOMEUIINFO_MENU_CLEAR_ITEM (clear_cb, NULL),
+	GNOMEUIINFO_ITEM_NONE (N_("R_emove All"),
+			       N_("Remove all items"),
+			       clear_cb),
 	GNOMEUIINFO_MENU_SELECT_ALL_ITEM (select_all_cb, NULL),
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo help_menu[] = {
-	GNOMEUIINFO_HELP ("meat-grinder"),
+	GNOMEUIINFO_HELP ("archive-generator"),
 	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
 	GNOMEUIINFO_END
 };
@@ -837,6 +962,7 @@ init_gui (void)
 	GtkWidget *vbox;
 	GtkWidget *sw;
 	GtkWidget *w;
+	GtkWidget *image;
 	GtkTooltips *tips;
 
 	tips = gtk_tooltips_new ();
@@ -849,12 +975,12 @@ init_gui (void)
 	gtk_window_set_resizable (GTK_WINDOW (app), TRUE);
 
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/document-icons/gnome-compressed.png");
+        
+	gnome_app_create_menus (GNOME_APP (app), grinder_menu);
 
         g_signal_connect (G_OBJECT (app), "delete_event",
 			  G_CALLBACK (quit_cb), NULL);
 
-	/*set up the menu*/
-        gnome_app_create_menus (GNOME_APP (app), grinder_menu);
 
 	vbox = gtk_vbox_new (FALSE, 5);
 	gtk_widget_show (vbox);
@@ -893,15 +1019,10 @@ init_gui (void)
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
 	/* tarball icon */
-	compress_button = gtk_button_new ();
+	compress_button = archive_button_new_with_image (_("_Create Archive"), 
+							 compress_icon);
 	gtk_widget_show (compress_button);
-	w = NULL;
-	if (compress_icon != NULL)
-		w = gtk_image_new_from_file (compress_icon);
-	if (w == NULL)
-		w = gtk_label_new (_("Archive"));
-	gtk_widget_show (w);
-	gtk_container_add (GTK_CONTAINER (compress_button), w);
+
 	gtk_box_pack_start (GTK_BOX (hbox), compress_button, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (compress_button), "clicked",
 			  G_CALLBACK (archive_cb), NULL);
@@ -915,6 +1036,7 @@ init_gui (void)
 	gtk_widget_set_sensitive (file_menu[1].widget, FALSE);
 	gtk_widget_set_sensitive (edit_menu[0].widget, FALSE);
 	gtk_widget_set_sensitive (edit_menu[1].widget, FALSE);
+	gtk_widget_set_sensitive (edit_menu[2].widget, FALSE);
 
 	/* setup dnd */
 	gtk_drag_source_set (compress_button,
@@ -945,6 +1067,19 @@ init_gui (void)
 
 	gnome_app_set_contents (GNOME_APP (app), vbox);
 
+        app_bar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
+
+	gnome_app_set_statusbar (GNOME_APP (app), app_bar);
+        progress_bar = GTK_WIDGET (gnome_appbar_get_progress (
+                                GNOME_APPBAR (app_bar)));
+        g_object_set (G_OBJECT (progress_bar),
+                      "pulse_step", 0.1,
+                      NULL);
+	
+	/*set up the menu*/
+	gnome_app_install_menu_hints (GNOME_APP (app), grinder_menu);
+
+	gtk_widget_show_all (vbox);
 	gtk_widget_show (app);
 }
 
@@ -992,9 +1127,9 @@ main (gint argc, gchar *argv [])
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
-	gnome_program_init ("meat-grinder", VERSION, LIBGNOMEUI_MODULE,
-				    argc, argv, GNOME_PARAM_POPT_TABLE,
-				    &options,GNOME_PARAM_APP_DATADIR,DATADIR,NULL);
+	gnome_program_init ("archive-generator", VERSION, LIBGNOMEUI_MODULE,
+			    argc, argv, GNOME_PARAM_POPT_TABLE,
+			    &options,GNOME_PARAM_APP_DATADIR,DATADIR,NULL);
 
 	client = gnome_master_client ();
 
@@ -1003,9 +1138,6 @@ main (gint argc, gchar *argv [])
 
 	g_signal_connect (client, "die", G_CALLBACK (die), NULL);
 
-	/* no icon yet */
-	/*gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-meat-grinder.png");*/
-
 	file_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "mc/i-regular.png", TRUE, NULL);
 	if (file_icon == NULL)
 		file_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "nautilus/i-regular.png", TRUE, NULL);
@@ -1026,7 +1158,7 @@ main (gint argc, gchar *argv [])
 	if (folder_icon == NULL)
 		folder_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-unknown.png", TRUE, NULL);
 
-	compress_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-compressed.png", TRUE, NULL);
+	compress_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "document-icons/gnome-compressed.png", TRUE, NULL);
 	if (compress_icon == NULL)
 		compress_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "nautilus/gnome-compressed.png", TRUE, NULL);
 	if (compress_icon == NULL)
@@ -1067,6 +1199,7 @@ main (gint argc, gchar *argv [])
 	create_archive_widget = file_menu[1].widget;
 	remove_widget = edit_menu[0].widget;
 	clear_widget = edit_menu[1].widget;
+	selectall_widget = edit_menu[2].widget;
 
 	file_ht = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -1099,3 +1232,4 @@ main (gint argc, gchar *argv [])
 	
 	return 0;
 }
+
